unit rp_Functions;

{-------------------------------------------------------------------------------------------
Stand 23.02.2021
--------------------------------------------------------------------------------------------}

{$mode objfpc}{$H+}

interface

uses
  Classes, Controls, ComCtrls, SysUtils, Windows, Graphics, Grids, DateUtils, Process;


Type CharSet = set of Char;

     tArr = record
       iA,iE : Integer;
       s     : String;
     end;

     rbReturnCode = (rbOk,rbRunError,rbNoBatchFound);

Const chrQuote : Char = #39;


Function RunBatch(BatchFile: String; var tslReturn: TStringList): rbReturnCode;

Function GetNextPosOf(s: String; Sign: Char; iStart: Integer): Integer;
Function ParseStr(pcs: PChar; iNo: Integer; delim : Charset): String;
Function ParsePChar(pcs: PChar; iNo: Integer; delim : Charset): PChar;
Function ParsePCharCount(pcs: PChar; delim: CharSet): LongInt;
function Parse(Char, S: string; Count: Integer): string;
Function ParseCount(Char, S: string): Integer;
Function ParseIndexOf(Char, sSearch, sSearchIn: String): Integer;

// Funktionen um ganze Wörter in Strings zu identifizieren
// Function SepariereWorteInString(s: String; cDelim: string): string; alt
Function StringToToken(cDelim: string; sText: String): string;
function CountTokenInString(cDelim: String; s: String): Integer;
function ParseTokenFromString(cDelim: String; s: String; i: Integer): String;
Function StringInToken(cDelim: String; sSearch: string; sTokenString: String): Integer;


Function Str2Int(s: String; Initial: Integer): Integer;
Function StrToDouble(str: String): Double;
Function IntToStrZero(w: Integer;len: Integer): String;
Function StrToHex(const S: String): String;
Function HexToStr(s: String): String;
Function IsNumeric(s: string): Boolean;
Function FormatString(Text, Format: String): String;
Function ConvertToInt(s: String; Initial: Integer): Integer;

function TrendAnfangswert(arr: Array of double; n : Integer): double;
function TrendLineareVeraenderung(arr: Array of double; n : Integer): double;

function FileInUse(const FileName: string): boolean;

Function SortByValue(Item1,Item2: Pointer): Integer;

function  GetTextwidth(const T : String; Font : TFont ) : Integer;
procedure GridColWidth(grd: TStringGrid;min,max:word; col, plus: integer);
procedure PutCtrlInPanel(wCtrl: TWincontrol; StatusBar: TStatusBar; PANEL_ID: Integer);
function  TrueFontWidth(fnt: TFont; const text:string): Integer;

Procedure Datei_sperren(Datei: String);
Procedure Datei_entsperren(Datei: String);

Function GetWeekNo(dt: TDateTime;MondayIsFirstDay: Boolean): Integer;
function IsLeapYear(dt: TDateTime): Boolean;
Function GetDayName(Date: TDateTime): String;

function Kalenderwoche(Tag, Monat: Byte; Jahr : Word) : Byte;
function Kalendertag(tag, monat : byte; jahr : word) : Word;
function Wochentag(t, m : Byte; j : Word) : Byte;
function Schaltjahr(wrdJahr : Word) : Boolean;

Function Neujahr(wrdJahr: word): TDate;
Function HeiligDreiKoenige(wrdJahr: word): TDate;
Function Karfreitag(wrdJahr: word): TDate;
function Ostersonntag(wrdJahr: word): TDate;
Function OsterMontag(wrdJahr: word): TDate;
Function TagDerArbeit(wrdJahr: word): TDate;
Function ChristiHimmelfahrt(wrdJahr: word): TDate;
Function PfingstSonntag(wrdJahr: word): TDate;
Function PfingstMontag(wrdJahr: word): TDate;
Function FronLeichnam(wrdJahr: word): TDate;
Function MariaHimmelfahrt(wrdJahr: word): TDate;
Function TagDerEinheit(wrdJahr: word): TDate;
Function Reformationstag(wrdJahr: word): TDate;
Function AllerHeiligen(wrdJahr: word): TDate;
function BussBettag(wrdJahr: word):TDate;
function Advent1(wrdJahr: word):TDate;
function Advent2(wrdJahr: word):TDate;
function Advent3(wrdJahr: word):TDate;
function Advent4(wrdJahr: word):TDate;
Function HeiligAbend(wrdJahr: word): TDate;

Function Rosenmontag(wrdJahr: word): TDate;
Function Fastnacht(wrdJahr: word): TDate;
Function Aschermittwoch(wrdJahr: word): TDate;
Function Muttertag(wrdJahr: word): TDate;
Function Silvester(wrdJahr: word): TDate;

procedure ListViewResize(TL: TListView);

implementation

procedure ListViewResize(TL: TListView);
var i, w, w2, wNeu : Integer;
begin
  w  := 0;
  w2 := tl.Width;
  for i := 1 to tl.Columns.Count -1 do
      w := w + tl.Columns.Items[i].Width;
  wNeu := w2 - w - 25;
  tl.Columns.Items[0].Width := wNeu;
end;


{ Konstanten-Array mit Anzahl Tagen je Monat, Index ist Monats-Nummer}
const
   m_DaysPerMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

Procedure Datei_sperren(Datei: String);
var
  lf_FileSize: Integer;
  lf_FileHandle: THandle;
begin
  lf_FileHandle := CreateFile(PChar(Datei), GENERIC_WRITE, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  lf_FileSize:= GetFileSize(lf_FileHandle, nil);
  LockFile(lf_FileHandle, 0, 0, lf_FileSize, 0);
  CloseHandle(lf_FileHandle);
end;

Procedure Datei_entsperren(Datei: String);
var
  lf_FileSize: Integer;
  lf_FileHandle: THandle;
begin
  lf_FileHandle := CreateFile(PChar(Datei), GENERIC_WRITE, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  lf_FileSize:= GetFileSize(lf_FileHandle, nil);
  UnlockFile(lf_FileHandle, 0, 0, lf_FileSize, 0);
  CloseHandle(lf_FileHandle);
end;

Function ConvertToInt(s: String; Initial: Integer): Integer;
begin
  if TryStrToInt(s,result) = false then result := Initial;
end;

Function GetDayName(Date: TDateTime): String;
begin
  case DayOfWeek(Date) of
    1 : result := 'Sonntag';
    2 : result := 'Montag';
    3 : result := 'Dienstag';
    4 : result := 'Mittwoch';
    5 : result := 'Donnerstag';
    6 : result := 'Freitag';
    7 : result := 'Samstag';
  end;
end;


Function GetWeekNo(dt: TDateTime;MondayIsFirstDay: Boolean): Integer;
var m_CurrentDay, m_CurrentYear, m_CurrentMonth : Word;
    i, DaysSinceFirst: Word;
    iWeeks : Word;
    DaysInFirstWeek : Word;
    RestOfDiv : Word;
begin
  iWeeks := 0;                                                     { Rückgabewert initialisieren}
  DecodeDate(dt,m_CurrentYear,m_CurrentMonth,m_CurrentDay);        { Datumswert in seine Bestandteile zerlegen (JJ,MM,TT) }
  DaysInFirstWeek := 7-DayOfWeek(EncodeDate(m_CurrentYear,01,01))+1; { Anzahl der Tage in Woche 1 }
  If MondayIsFirstDay then Inc(DaysInFirstWeek);                   { DayOfWeek setzt Sonntag als 1. Tag der Woche }
  DaysSinceFirst := 0;                                             { Tagezähler initialisieren }
  For i := 1 to m_CurrentMonth-1 do                                { Anzahl der Tage seit dem 1.1. für alle Monate ermitteln}
  begin
    Inc(DaysSinceFirst,m_DaysPerMonth[i]);
    if i = 2 then
       if IsLeapYear(dt) then inc(DaysSinceFirst,1);             { Schaltjahr berücksichtigen }
  end;
  inc(DaysSinceFirst,m_CurrentDay);
  if DaysSinceFirst <= DaysInFirstWeek then
  begin
     iWeeks := 1;
  end
  else
  begin
    RestOfDiv := 0;
    dec(DaysSinceFirst,DaysInFirstWeek);                              { Anzahl Tage im aktuelle Monat addieren }
    RestOfDiv := DaysSinceFirst mod 7;
    iWeeks := DaysSinceFirst div 7;
    if RestOfDiv > 0 then inc(iWeeks);
    if (DaysInFirstWeek > 4) then inc(iWeeks);
  end;
  result := iWeeks;
end;

{Diese Funktion prüft, ob das aktuelle Jahr ein Schaltjahr ist}
function IsLeapYear(dt: TDateTime): Boolean;
var m_CurrentDay, m_CurrentYear, m_CurrentMonth : Word;
begin
  {Datumswert in seine Bestandteile zerlegen (JJ,MM,TT) }
  DecodeDate(dt,m_CurrentYear,m_CurrentMonth,m_CurrentDay);
  Result := ( m_CurrentYear mod 4 = 0 ) and
            ( ( m_CurrentYear mod 100 <> 0 ) or ( m_CurrentYear mod 400 = 0 ) );
end;

Function StringToToken(cDelim: string; sText: String): string;
var strAllowed : String;
    i : Integer;
begin
  strAllowed := 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789üÜäÄöÖß_-';

  result := '';
  if length(sText) = 0 then Exit;
  sText := trim(sText);

  for i := 1 to length(sText) do
  begin
    if pos( sText[i], strAllowed) > 0 then                             // gesucht, durchsucht
      result := result + sText[i]
    else
      result := result + cDelim;
  end;
  while pos(cDelim+cDelim,result) > 0 do result := StringReplace(result, cDelim+cDelim, cDelim,[rfReplaceAll]);

end;

Function StringInToken(cDelim: String; sSearch: string; sTokenString: String): Integer;
begin
  result := 0;
  result := pos(sSearch,sTokenString);
  if result = 1 then
     result := pos(sSearch+chr(4),sTokenString)
  else
     result := pos(chr(4)+sSearch+chr(4),sTokenString);
end;

function ParseTokenFromString(cDelim: String; s: String; i: Integer): String;
begin
  result := '';
  s := StringToToken(s,cDelim);
  if i <= ParseCount(cDelim,s) then result := Parse(cDelim,s,i);
end;

function CountTokenInString(cDelim: String; s: String): Integer;
begin
  result := 0;
  s := StringToToken(s,cDelim);
  result := ParseCount(cDelim,s);
end;


// SetLength(Text, Length(Text) -1));

Function GetNextPosOf(s: String; Sign: Char; iStart: Integer): Integer;
var i, iLen: Integer;
begin
  result := -1;
  iLen := length(s);
  if iLen > iStart then
  begin
    i := iStart;
    repeat
      if s[i] = Sign then
      begin
        result := i;
        break;
      end;
      inc(i);
    until (i > iLen);
  end;
end;

Function ParsePCharCount(pcs: PChar; delim: CharSet): LongInt;
var len : PChar;
    i : Integer;
begin
  len := pcs;
  result := 0;
  i := 0;
  if len[i] <> #0 then
  begin
    while Len[i] <> #0 do
    begin
      if len[i] in delim then inc(result);
      inc(i);
    end;
    inc(result);
  end
  else
    result := 0;

end;

Function ParsePChar(pcs: PChar; iNo: Integer; delim : Charset): PChar;
var len : PChar;
begin
  len := pcs;                                              { zweiter Pointer }
  while (len[0] in delim) and (len[0] <> #0) do inc(len);
  result := len;
  if len[0] <> #0 then
  begin
    while (not (len[0] in delim)) and (len[0] <> #0) do inc(len);
    if iNo > 1 then result := ParsePChar(len+1,iNo-1,delim)
    else            strLcopy(len,#0,1);
  end
  else
     result := nil;
end; { Function }

Function ParseStr(pcs: PChar; iNo: Integer; delim : Charset): String;
var pStart, len : PChar;
begin
  len := pcs;                                              { zweiter Pointer }
  while (len[0] in delim) and (len[0] <> #0) do inc(len);
  PStart := len;
  if len[0] <> #0 then
  begin
    while (not (len[0] in delim)) and (len[0] <> #0) do inc(len);
    if iNo > 1 then {pStart := }ParsePChar(len+1,iNo-1,delim)
    else
      begin
        strLcopy(len,#0,1);
        result := StrPas(pStart);
      end;
  end
  else
     result := '';
end; { Function }

Function ParseIndexOf(Char, sSearch, sSearchIn: String): Integer;
var i : Integer;
begin
  result := -1;
  for i := 1 to ParseCount(Char,sSearchIn) do
  begin
    if Parse(Char,sSearchIn,i) = sSearch then
    begin
      result := i;
      break;
    end;
  end;
end;

Function Str2Int(s: String; Initial: Integer): Integer;
begin
  if TryStrToInt(s,result) = False then result := Initial;
end;

Function StrToDouble(str: String): Double;
var code : Integer;
begin
  code := pos(',',str);
  if code > 0 then str[code] := '.';
  val(str,result,code);
  if code <> 0 then result := 0.0;
end;

Function IntToStrZero(w: Integer;len: Integer): String;
var s: String;
    i : Integer;
begin
  Str(w:len,s);
  for i := 1 to length(s) do if s[i] = ' ' then s[i] := '0';
  result := s;
end;

function TrendAnfangswert(arr: Array of double; n : Integer): double;
var i : Integer;
    sXi, sYi, sXiYi : double;
begin
  sXi := 0.0;
  sYi := 0.0;
  sXiYi := 0.0;
  { sXi }   for i := 0 to n-1 do sXi := sXi + i;
  { sYi }   for i := 0 to n-1 do sYi := sYi + arr[i];
  { sXiYi } for i := 0 to n-1 do sXiYi := sXiYi + (arr[i]*i);

  TrendAnfangswert := ( (((sXi * sXi)*sYi) - (sXi * sXiYi)) / ((sXi * sXi) - (n * (sXi * sXi))) ) * (-1);
end;

function TrendLineareVeraenderung(arr: Array of double; n : Integer): double;
var i : Integer;
    sXi, sYi, sXiYi : double;
begin
  sXi := 0.0;
  sYi := 0.0;
  sXiYi := 0.0;
  { sXi }   for i := 0 to n-1 do sXi := sXi + i;
  { sYi }   for i := 0 to n-1 do sYi := sYi + arr[i];
  { sXiYi } for i := 0 to n-1 do sXiYi := sXiYi + (arr[i]*i);

  TrendLineareVeraenderung := ((sXi * sYi) - (n * sXiYi)) / ( (sXi * sXi) - (n * (sXi * sXi)) );

end;

procedure TrendGdUngerade(var Data: Array of double; var TrendData: Array of Double; PeriodenSchnitt: Integer);
var iArea    : Integer; {Bereichsvariable für Trendwert}
    iData    : Integer; {Zeiger auf Trendposition  }
    Value    : Double;
    i, iDataStart, iDataEnde : Integer;
begin
  iArea := PeriodenSchnitt DIV 2;
  iDataStart := iArea+1;
  iDataEnde  := High(Data)-iArea-1;
  For iData := iDataStart to iDataEnde do
  begin
    Value := 0.0;
    For i := (iData-iArea) to (iData+iArea) do Value := Value + Data[i];
    TrendData[iData] := Value / Periodenschnitt;
  end;
end;

Procedure TrendGdGerade(var Data: Array of double; var ResultData: Array of Double; PeriodenSchnitt: Integer);
var iArea    : Integer; {Bereichsvariable für Trendwert}
    iData    : Integer; {Zeiger auf Trendposition  }
    Value1, Value2    : Double;
    i, iDataStart, iDataEnde : Integer;
begin
  iArea := PeriodenSchnitt DIV 2;
  iDataStart := iArea+1;
  iDataEnde  := High(Data)-iArea;
  Value1 := 0.0;
  Value2 := 0.0;
  For iData := iDataStart to iDataEnde-1 do
  begin
    if Value1 = 0.0 then For i := (iData-iArea-1) to (iData+iArea-1) do Value1 := Value1 + Data[i];
    For i := (iData-iArea)   to (iData+iArea)   do Value2 := Value2 + Data[i];
    Value1 := Value1 / Periodenschnitt;
    Value2 := Value2 / Periodenschnitt;
    ResultData[iData] := (Value1 + Value2) / 2;
    Value1 := Value2;
  end;
end;

Function TrendGleitenderDurchschnitt(Data: Array of double; var TrendData: Array of Double; PeriodenSchnitt: Integer): Word;
begin
  result := 100;
  if Periodenschnitt mod 2 > 0 then
  begin
     TrendGdUngerade(Data,TrendData,PeriodenSchnitt);
     result := 101;
  end
  else
     TrendGdGerade(Data,TrendData,PeriodenSchnitt);
end;

Procedure TrendWochenschnitt(var Data: Array of double; var ResultData: Array of Double; PeriodenSchnitt: Integer);
var iArea    : Integer; {Bereichsvariable für Trendwert}
    iData    : Integer; {Zeiger auf Trendposition  }
    Value    : Double;
    i, iDataStart, iDataEnde : Integer;
begin
  iArea := PeriodenSchnitt;
  iDataStart := iArea-1;
  iDataEnde  := High(Data);
  For iData := iDataStart to iDataEnde do
  begin
    Value := 0.0;
    For i := (iData-iArea) to iData do Value := Value + Data[i];
    ResultData[iData] := Value / Periodenschnitt;
  end;
end;

Function IndexDataToRef(var Data: Array of double; ReferenzData: Array of double; var ResultData: Array of Double; Basis: Integer): Integer;
var i : Integer;
begin
  result := -1;
  Try
    For i := 0 to HIGH(Data) do
      ResultData[i] := Data[i] * Basis / ReferenzData[i];
    Result := 0
  except
  end; {Try}
end;

function FileInUse(const FileName: string): boolean;
var
   Success: Cardinal;
begin
   Result := false;
   if FileExists(FileName) then
   begin
     Success := CreateFile(PChar(FileName),
       GENERIC_READ, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
     Result := Success = INVALID_HANDLE_VALUE;
     if not Result then
       CloseHandle(Success);
   end;
end;

Function CountSign(Text : String): Integer;
const sAllowed : String = '1234567890#';
var i : Integer;
begin
  result := 0;
  for i := 0 to length(Text) do
      if pos(Text[i],sAllowed) > 0 then
         inc(result);
end;

Function FormatString(Text, Format: String): String;
var i : Integer;
begin
  result := '';
  if CountSign(Text) <> CountSign(Format) then exit;
  for i := 1 to length(Format) do
  begin
    if Format[i] = '#' then
       result := result + Text[i]
    else
       result := result + Format[i];
  end;
end;

function StrToHex(const S: String): String;
const
  HexDigits: array[0..15] of Char = '0123456789ABCDEF';
var
  I: Integer;
  P1: PChar;
  P2: PChar;
  B: Byte;
begin

  SetLength(Result, Length(S) * 2);
  P1 := @S[1];
  P2 := @Result[1];

  for I := 1 to Length(S) do
  begin
    B := Byte(P1^);
    P2^ := HexDigits[B shr 4];
    Inc(P2);
    P2^ := HexDigits[B and $F];
    Inc(P1);
    Inc(P2);
  end;

end;

Function HexToStr(s: String): String;
Var i: Integer;
Begin
  Result:=''; i:=1;
  While i<Length(s) Do Begin
    Result:=Result+Chr(StrToIntDef('$'+Copy(s,i,2),0));
    Inc(i,2);
  End;
End;

Function IsNumeric(s: string): Boolean;
var Zahl: Integer;
begin
   IsNumeric := TryStrToInt(s,Zahl);
end;
Function SortByValue(Item1,Item2: Pointer): Integer;
begin
  if (Double(Item1^) < Double(Item2^)) then result := -1
  else
     if (Double(Item1^) > Double(Item2^)) then result := 1
     else
        result := 0;
end;

Function ParseCount(Char, S: string): Integer;
var i : Integer;
begin
  result := 0;
  if length(s) = 0 then exit;
  for i := 1 to length(s) do
  begin
    if s[i] = Char then inc(result);
  end;
  if s[i] <> Char then inc(result);
end;

function Parse(Char, S: string; Count: Integer): string;
  var
   I: Integer;
   T: string;
 begin
   result := '';
   if length(s) = 0 then exit;
   if S[Length(S)] <> Char then
     S := S + Char;
   for I := 1 to Count do
   begin
     TRY
       T := Copy(S, 0, Pos(Char, S) - 1);
       S := Copy(S, Pos(Char, S) + 1, Length(S));
     except
       exit;
     end;
   end;
   Result := T;
end;

function GetTextwidth(const T : String; Font : TFont ) : Integer;
var
   Canvas : TCanvas;
begin
   Canvas := TCanvas.Create;
   Try
     Canvas.Handle := GetDC( 0 );
     Try
     Canvas.Font := Font;
     Result := Canvas.TextWidth(T);
     Finally
       ReleaseDC( 0, Canvas.Handle );
     end;
   Finally
     Canvas.Free;
   end;
end;

function TrueFontWidth(fnt: TFont; const text:string): Integer;
var
   dc: hdc;
   tsize : Windows.TSize;
begin
   dc := GetDC(0);
   SelectObject(DC, fnt.Handle);
   GetTextExtentPoint32(dc, PChar(text), Length(text), tsize);
   ReleaseDC(0, DC);
   Result := tsize.cx;
end;

procedure GridColWidth(grd:TStringGrid;min,max:word; col, plus: integer);
var
   Zeile,Spalte,tmp,len: word;
begin
   with grd do begin // für alle Spalten
     for Spalte:=0 to col do begin
       len:=0;
       for Zeile:=0 to (RowCount-1) do begin // für alle Zeilen
         tmp:=Canvas.TextWidth(Cells[Spalte,Zeile]);
         if tmp>len then len:=tmp // neue max Breite
       end; // for Zeile
       if max>0 then if len>max then len:=max; // Maximalbreite
       if len<min then len:=min; // Minimalbreite
       ColWidths[Spalte]:=len+GridLineWidth+5+plus // Spalte verbreitern
     end // for Spalte
   end // with
end;

procedure PutCtrlInPanel(wCtrl: TWincontrol; StatusBar: TStatusBar; PANEL_ID: Integer);
var
  ARect: TRect;
begin
//  StatusBar.Perform((WM_USER + 10), PANEL_ID, Integer(@ARect));
//  wCtrl.Parent := StatusBar;
//  ARect.Top:=ARect.Top+1;
//  ARect.Left:=ARect.Left+1;
//  ARect.Height:=ARect.Height-1;
//  ARect.Width:=ARect.Width-1;
//  wCtrl.BoundsRect := ARect;

  StatusBar.Perform((WM_USER + 10), PANEL_ID, Integer(@ARect));
  wCtrl.Parent := StatusBar;
  wCtrl.BoundsRect := ARect;
end;

procedure PutCtrlInSatusbar(wCtrl: TWincontrol; StatusBar: TStatusBar; PANEL_ID: Integer; iPanel: Integer);
var
  ARect: TRect;
  Panel: TStatuspanel;
begin
//  StatusBar.Perform((WM_USER + 10), PANEL_ID, Integer(@ARect));
//  wCtrl.Parent := StatusBar;
//  ARect.Top:=ARect.Top+1;
//  ARect.Left:=ARect.Left+1;
//  ARect.Height:=ARect.Height-1;
//  ARect.Width:=ARect.Width-1;
//  wCtrl.BoundsRect := ARect;

  Panel := Statusbar.Panels.Items[iPanel];

  StatusBar.Perform((WM_USER + 10), PANEL_ID, Integer(@ARect));
  wCtrl.Parent := StatusBar;
  wCtrl.BoundsRect := ARect;
end;

{ Datumsfunktionen }
function Schaltjahr(wrdJahr : Word) : Boolean;
Begin
     result := ( (wrdJahr mod 4 = 0) and not (wrdJahr mod 100 = 0) ) or (wrdJahr mod 400 = 0);
end;

function Wochentag(t, m : Byte; j : Word) : Byte;
const
      c1 : ARRAY[1..12] OF Byte = (0,3,3,6,1,4,6,2,5,0,3,5);
      c2 : ARRAY[0..3] OF Byte = (6,4,2,0);
var
    a,w, j1, j2 : Byte;
begin
     j1 := j div 100;
     j2 := j mod 100;
     a:= (t MOD 7) + (c1[m]) + (j2 MOD 7) + ((j2 DIV 4) MOD 7) + (c2[(j1 MOD 4)]);
     IF Schaltjahr(j) THEN Dec(a);
     w := a MOD 7;
     Wochentag := w;
end;

function Kalendertag(tag, monat : byte; jahr : word) : Word;
const
      TageProMonat : ARRAY[1..11] OF Byte =(31,28,31,30,31,30,31,31,30,31,30);
var
    counter : Byte; GesamtTage : Word;
begin
     GesamtTage:=0;
     FOR counter := 1 TO (monat-1) DO GesamtTage := GesamtTage + TageProMonat[counter];
     GesamtTage := GesamtTage + tag;
     IF Schaltjahr(jahr) AND (monat > 2) THEN Inc(GesamtTage);
     Kalendertag := GesamtTage;
end;

function Kalenderwoche(Tag, Monat: Byte; Jahr : Word) : Byte;
var
    Kalendertage, Vorjahr : Word;
    Woche, Primus : Byte;
const
    Korrektur : ARRAY[0..6,1..2] OF ShortInt = ((-6,0),(0,1),(-1,1),(-2,1),(-3,1),(-4,0),(-5,0));
begin
     kalendertage:=kalendertag(tag,monat,jahr);
     Vorjahr := Jahr - 1;
     Primus:=Wochentag(1,1,Jahr);
     Woche:=Trunc((Kalendertage - 1 - Korrektur[Primus,1]) / 7) + Korrektur[Primus,2];
     IF (Woche = 53) THEN
     BEGIN
       IF (Primus = 4) OR (Wochentag(31,12, Jahr) = 4) THEN Woche := 53
                                                        ELSE Woche := 1;
     END;
     IF (Woche=0) THEN
     BEGIN
       IF ((Wochentag(31,12, Vorjahr) = 4) OR (Wochentag(1,1, Vorjahr) = 4))
         THEN Woche := 53 ELSE Woche := 52;
     END;
     Kalenderwoche:=Woche;
end;

Function Neujahr(wrdJahr: word): TDate;
begin
  result := encodedate(wrdJahr,1,1);
end;

Function Rosenmontag(wrdJahr: word): TDate;
begin
  result := Ostersonntag(wrdJahr) - 48;
end;

Function Fastnacht(wrdJahr: word): TDate;
begin
  result := Ostersonntag(wrdJahr) - 47;
end;

Function Aschermittwoch(wrdJahr: word): TDate;
begin
  result := Ostersonntag(wrdJahr) - 46;
end;

function Ostersonntag(wrdJahr: word): TDate;
  // Osterfunktion nach Carl Friedrich Gauß (1800). Rückgabewert
  // ist das Datum des Ostersonntags im angegebenen Jahr
  // Gültigkeitsbereich: 1583 - 8702
var
  a: integer;
  b: integer;
  c: integer;
  d: integer;
  e: integer;
  f: integer;
  wrdTage: word;
  wrdMonat: word;

begin

  // Prüft, ob das Jahr korrekt ist
  if wrdJahr < 1583 then
  begin
    Exit;
  end;

  if wrdJahr > 8702 then
  begin
    Exit;
  end;

  // Die Gauss-Formel
  a := wrdJahr mod 19;
  b := wrdJahr div 100;
  c := (8 * b + 13) div 25 - 2;
  d := b - (wrdJahr div 400) - 2;
  e := (19 * (wrdJahr mod 19) + ((15 - c + d) mod 30)) mod 30;

  if e = 28 then
    if a > 10 then
      e := 27
    else if e = 29 then
      e := 28;

  f := (d + 6 * e + 2 * (wrdJahr mod 4) + 4 * (wrdJahr mod 7) + 6) mod 7;

  if (e + f + 22) > 31 then
  begin
    wrdMonat := 4;
    wrdTage := ((e + f + 22) - 31);
  end
  else
  begin
    wrdMonat := 3;
    wrdTage := (e + f + 22);
  end;

  // Rückgabewert als Datum bereitstellen
  Result := EncodeDate(wrdJahr, wrdMonat, wrdTage);

end;

Function OsterMontag(wrdJahr: word): TDate;
begin
  result := OsterSonntag(wrdJahr) + 1;
end;

// Gibt das Tagesdatum für den Pfingstsonntag eines Jahres zurück
Function PfingstSonntag(wrdJahr: word): TDate;
begin
  result := OsterSonntag(wrdJahr) + 49
end;

// Gibt das Tagesdatum für den Pfingstmontag eines Jahres zurück
Function PfingstMontag(wrdJahr: word): TDate;
begin
  result := OsterSonntag(wrdJahr) + 49 + 1
end;

// Gibt das Tagesdatum für Fronleichnahm eines Jahres zurück
Function FronLeichnam(wrdJahr: word): TDate;
begin
  result := OsterSonntag(wrdJahr) + 60
end;

// Gibt das Tagesdatum für Christi Himmelfahrt eines Jahres zurück
Function ChristiHimmelfahrt(wrdJahr: word): TDate;
begin
  result := OsterSonntag(wrdJahr) + 39
end;

Function HeiligDreiKoenige(wrdJahr: word): TDate;
begin
  result := encodedate(wrdJahr,1,6);
end;

Function TagDerArbeit(wrdJahr: word): TDate;
begin
    result := encodedate(wrdJahr,5,1);
end;

Function TagDerEinheit(wrdJahr: word): TDate;
begin
    result := encodedate(wrdJahr,10,3);
end;

Function Reformationstag(wrdJahr: word): TDate;
begin
    result := encodedate(wrdJahr,10,31);
end;

Function Karfreitag(wrdJahr: word): TDate;
begin
    result := OsterSonntag(wrdJahr)-2;
end;

//Mariä Himmelfahrt
Function MariaHimmelfahrt(wrdJahr: word): TDate;
begin
    result := encodedate(wrdJahr,8,15);
end;

Function AllerHeiligen(wrdJahr: word): TDate;
begin
    AllerHeiligen := encodedate(wrdJahr,11,1);
end;

// 1. Weihnachtstag
Function HeiligAbend(wrdJahr: word): TDate;
begin
    HeiligAbend := encodedate(wrdJahr,12,24);
end;

function Advent4(wrdJahr: word):TDate;
var advent:TDate;
begin
  // Datum des Heiligen Abends; 24.12
  advent := HeiligAbend(wrdJahr);
  // 4. Advent ermitteln; Sonntag vor dem 1.Weihnachtsfeiertag
  while not (DayOfTheWeek(advent)=7) do advent:=advent-1.0;
  //so, zum Schluss noch 32 Tage abziehen
  Result:=advent;
end; // BussBettag

function Advent3(wrdJahr: word):TDate;
begin
  Result:=Advent4(wrdJahr)-7;
end; // BussBettag

function Advent2(wrdJahr: word):TDate;
begin
  Result:=Advent3(wrdJahr)-7;
end; // BussBettag

function Advent1(wrdJahr: word):TDate;
begin
  Result:=Advent2(wrdJahr)-7;
end; // BussBettag

function BussBettag(wrdJahr: word):TDate;
var advent:TDate;
begin
  Result:=Advent4(wrdJahr)-32;
end; // BussBettag

Function Muttertag(wrdJahr: word): TDate;
begin
  result := encodedate(wrdJahr,5,15 - Wochentag(1,5,wrdJahr));
end;

Function Silvester(wrdJahr: word): TDate;
begin
  result := encodedate(wrdJahr,12,31);
end;

//      rbReturnCode = (rbOk,rbRunError,rbNoBatchFound);

Function RunBatch(BatchFile: String; var tslReturn: TStringList): rbReturnCode;
var
  AProcess: TProcess;
begin

  // Jetzt erzeugen wir das TProcess Objekt und ordnen es der Variablen AProcess zu.
  AProcess := TProcess.Create(nil);

  // Gibt an, welcher Befehl vom Prozess ausgeführt werden soll
  AProcess.CommandLine := BatchFile;

  // Wir definieren eine Option, wie das Programm ausgeführt werden soll. Dies stellt sicher, dass unser Programm nicht vor Beendigung des aufgerufenen
  // Programmes fortgesetzt wird. Außerdem geben wir an, dass wir die Ausgabe lesen wollen
  AProcess.Options := AProcess.Options + [poWaitOnExit, poUsePipes];
  AProcess.ShowWindow := swoHIDE;

  // Startet den Prozess, nachdem die Parameter entsprechend gesetzt sind
  AProcess.Execute;

  // Die Ausgabe wird nun in die Stringliste gelesen
  tslReturn.LoadFromStream(AProcess.Output);

  AProcess.Free;
end;



end.



