unit ulElements;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LConvEncoding, uTypes, Math;

const
  ElementFileCSV = '}Elements.dim';
  ElementFileStream = 'Elements.lst';

type

{------------------------------------------------------------------------------}
{ Definition TM1-Element                                                       }
{ Quelle: Elements.dim                                                         }
{ Header: Vater;Kind;Attribute                                                 }
{ Satz:   ADMIN;ADMIN;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                      }
{         SecurityAdmin;SecurityAdmin;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      }
{------------------------------------------------------------------------------}


  tleMode = (tleBezeichnung,tleVater,tleKind,tleLevel,tleOrder);
  tleSort = (tleBezeichnungAsc,tleBezeichnungDesc,tleVaterAsc,tleVaterDesc,tleKindAsc,tleKindDesc,tleLevelAsc,tleLevelDesc,tleOrderAsc,tleOrderDesc);


  TElementRecord = record
    Key_Vater   : String;  { Key Vater-Element                                 }
    Key_Kind    : String;  { Key Kind-Element                                  }
    Bezeichnung : String;  { Bezeichnung Kind-Element                          }
    Level       : Integer; { Ebene                                             }
    Order       : Integer; { Index des Lesens                                  }
  end;

    PElementRecord = ^TElementRecord;

    TLElement = class(TList)
    private
      pElement      : ^TElementRecord;
      Stream     : TStream;
      Sorted     : tleMode;
      Datum      : TDateTime;   // Letztes Ladedatum mit Zeit
    protected
      function    AddData(value: TElementRecord): Integer;
      function    GetData(i: Integer): TElementRecord;
      procedure   SetData(i: Integer; value: TElementRecord);
    public
      constructor Create;
      destructor  Destroy; override;
      Procedure   ClearList;
      Procedure   LoadFromStream(Dimension: String);
      Procedure   SaveToStream(Dimension: String);
      Procedure   LoadFromCSV(Dimension: String);
      procedure   SortBy(Order: tleSort);
      property    Items[i: Integer]: TElementRecord read GetData;
      property    count;
    end;

  var lElements: TLElement;

implementation

uses ufmain, rp_Functions;

function CompareBezeichnungAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementRecord(Item1^).Bezeichnung, TElementRecord(Item2^).Bezeichnung);
end;

function CompareBezeichnungDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementRecord(Item2^).Bezeichnung, TElementRecord(Item1^).Bezeichnung);
end;


function CompareVaterAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementRecord(Item1^).Key_Vater, TElementRecord(Item2^).Key_Vater);
end;

function CompareVaterDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementRecord(Item2^).Key_Vater, TElementRecord(Item1^).Key_Vater);
end;

function CompareKindAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementRecord(Item1^).Key_Kind, TElementRecord(Item2^).Key_Kind);
end;

function CompareKindDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementRecord(Item2^).Key_Kind, TElementRecord(Item1^).Key_Kind);
end;

function CompareLevelAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TElementRecord(Item1^).Level, TElementRecord(Item2^).Level);
end;

function CompareLevelDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TElementRecord(Item2^).Level, TElementRecord(Item1^).Level);
end;

function CompareOrderAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TElementRecord(Item1^).Order, TElementRecord(Item2^).Order);
end;

function CompareOrderDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TElementRecord(Item2^).Order, TElementRecord(Item1^).Order);
end;

procedure   TLElement.SortBy(Order: tleSort);
begin
  case Order of
    tleBezeichnungAsc : Sort(@CompareBezeichnungAsc);
    tleBezeichnungDesc: Sort(@CompareBezeichnungDesc);
    tleVaterAsc       : Sort(@CompareVaterAsc);
    tleVaterDesc      : Sort(@CompareVaterDesc);
    tleKindAsc        : Sort(@CompareKindAsc);
    tleKindDesc       : Sort(@CompareKindDesc);
    tleLevelAsc       : Sort(@CompareLevelAsc);
    tleLevelDesc      : Sort(@CompareLevelDesc);
    tleOrderAsc       : Sort(@CompareOrderAsc);
    tleOrderDesc      : Sort(@CompareOrderDesc);
  end;
end;

constructor TLElement.Create;
begin
  inherited create;
end;

destructor TLElement.Destroy;
begin
  inherited Destroy;
end;

Procedure   TLElement.ClearList;
var p : PElementRecord;
begin
  while count > 0 do
  begin
    p := inherited Items[0];
    Dispose(p);
    Delete(0);
  end;
end;

function    TLElement.GetData(i: Integer): TElementRecord;
var p : PElementRecord;
begin
  p := inherited Items[i];
  result.Key_Vater   := p^.Key_Vater;
  result.Key_Kind    := p^.Key_Kind;
  result.Bezeichnung := p^.Bezeichnung;
  result.Level       := p^.Level;
  result.Order       := p^.Order;
end;

function    TLElement.AddData(value: TElementRecord): Integer;
begin
  new(pElement);
  pElement^.Key_Vater   := value.Key_Vater;
  pElement^.Key_Kind    := value.Key_Kind;
  pElement^.Bezeichnung := value.Bezeichnung;
  pElement^.Level       := value.Level;
  pElement^.Order       := value.Order;
  inherited Add(pElement);
  result := count-1;
end;

procedure   TLElement.SetData(i: Integer; value: TElementRecord);
var p : PElementRecord;
begin
  p := inherited Items[i];
  p^.Key_Vater   := value.Key_Vater;
  p^.Key_Kind    := value.Key_Kind;
  p^.Bezeichnung := value.Bezeichnung;
  p^.Level       := Value.Level;
  p^.Order       := Value.Order;
end;

Procedure   TLElement.LoadFromStream(Dimension: String);
var i, iLen, iCount: Integer;
    Data  : TElementRecord; // ein Zeiger auf die Datenstruktur
    sDatum : String;
begin

  clearlist;
  Stream := TFileStream.Create(fPathProg + Dimension + ExtLst, fmOpenRead);

  Stream.Read(iCount, SizeOf(iCount));

  Stream.Read(Sorted, SizeOf(Sorted));

  Stream.Read(Datum, SizeOf(Datum));

  For i := 1 to iCount do
  begin

    FillChar(Data,SizeOf(Data),0);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Key_Vater, iLen);
    Stream.Read(PChar(Data.Key_Vater)^, iLen);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Key_Kind, iLen);
    Stream.Read(PChar(Data.Key_Kind)^, iLen);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Bezeichnung, iLen);
    Stream.Read(PChar(Data.Bezeichnung)^, iLen);

    Stream.Read(Data.Level, SizeOf(Data.Level));
    Stream.Read(Data.Order, SizeOf(Data.Order));

    AddData(Data);
  end;
  stream.Free;

  if FileDateToDateTime(FileAge(fPathData + Dimension + ExtDim)) > Datum then LoadFromCSV(Dimension);
end;

procedure   TLElement.SaveToStream(Dimension: String);
var i, iLen, iCount: Integer;
    Data  : TElementRecord; // ein Zeiger auf die Datenstruktur
begin
  Stream := TFileStream.Create(fPathProg + Dimension + ExtLst, fmCreate);

  iCount := Count;

  Stream.Write(iCount, SizeOf(iCount));

  Stream.Write(Sorted, SizeOf(Sorted));

  Stream.Write(Datum, SizeOf(Datum));

  For i := 0 to iCount - 1 do
  begin
    Data := Items[i];
    iLen := Length(Data.Key_Vater);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Key_Vater)^, iLen);

    iLen := Length(Data.Key_Kind);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Key_Kind)^, iLen);

    iLen := Length(Data.Bezeichnung);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Bezeichnung)^, iLen);

    Stream.Write(Data.Level, SizeOf(Data.Level));
    Stream.Write(Data.Order, SizeOf(Data.Order));

  end;
  stream.Free;

end;

Procedure   TLElement.LoadFromCSV(Dimension: String);
var i, iVater, iKind, iBezeichnung, iLevel, iCount: Integer;
    Data  : TElementRecord;
    tsl: TStringList;
    s: String;
    sa: TStringArray;
begin


  iVater       := 0;
  iKind        := 0;
  iBezeichnung := 0;
  iCount       := 0;

  clearlist;
  tsl := TStringList.Create;

//  if fProgress <> NIL then fProgress.sb.Panels[0].Text:='Lade ' + Dimension + ExtDim;

  tsl.LoadFromFile(fPathData + Dimension + ExtDim);
  s := UpperCase(ConvertEncoding(tsl.Strings[0],GuessEncoding(tsl.Strings[0]), EncodingUTF8));

  i := tsl.Count;

  sa := s.Split(';');
  iCount := HIGH(sa);
  For i := 0 to iCount do
  begin
    if sa[i] = 'VATER'          then iVater := i;
    if sa[i] = 'KIND'           then iKind  := i;
    if sa[i] = 'BEZEICHNUNG(A)' then iBezeichnung  := i;
    if sa[i] = 'LEVEL(N)'       then iLevel        := i;
  end;

  Datum := FileDateToDateTime(FileAge(fPathData + Dimension + ExtDim));

  for i := 1 to tsl.Count-1 do
  begin
     s := ConvertEncoding(tsl.Strings[i],GuessEncoding(tsl.Strings[i]), EncodingUTF8);
     sa := s.Split(';');
     Data.Key_Vater   := sa[iVater];
     Data.Key_Kind    := sa[iKind];
     Data.Bezeichnung := sa[iBezeichnung];
     Data.Level       := StrToInt(sa[iLevel]);
     Data.Order       := i;
     AddData(Data);
  end;
  tsl.Clear;
  tsl.Free;
  setLength(sa,0);
  SaveToStream(Dimension);
end;


initialization
if lElements = NIL then
begin
  lElements := TlElement.Create;
end;

finalization
if lElements <> NIL then
begin
  lElements.ClearList;
  lElements.Free;
end;


end.

