unit ulElementSecurity;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, uTypes, LConvEncoding;

const
  ElementSecurityFileCSV = '}ElementSecurity_';
  ElementSecurityFileStream = 'ElementSecurity_';

type

{------------------------------------------------------------------------------}
{ Definition TM1-ElementSecurity                                               }
{ Quelle: ElementSecurity_zb_Kostenart.cub                                     }
{ Header: Vater;Kind;Bezeichnung                                               }
{ Satz:   Admin;Admin;                                                         }
{ Regel:  Wenn keine Bezeichnung dann ist Bezeichnung = Kind                   }
{------------------------------------------------------------------------------}

  tlesMode = (tlesElement,tlesGroup,tlesBoth);
  tlesSort = (tlesElementAsc,tlesElementDesc,tlesGroupAsc,tlesGroupDec,tlesBothAsc,tlesBothDesc);

  TElementSecurityRecord = record
    Element  : String;       { Element der Hierarchie                          }
    Group    : String;       { benutzergruppe                                  }
    Right    : String;       { Rechte                                          }
  end;

  PElementSecurityRecord = ^TElementSecurityRecord;

  TLElementSecurity = class(TList)
  private
    pElementSecurity      : ^TElementSecurityRecord;
    Sorted     : tlesSort;
    Stream     : TStream;
  protected
    function    GetData(i: Integer): TElementSecurityRecord;
    procedure   SetData(i: Integer; value: TElementSecurityRecord);
    procedure   SaveToStream(Dimension: String);
    Procedure   ClearList;
  public
    constructor Create;
    destructor  Destroy; override;
    function    AddData(value: TElementSecurityRecord): Integer;
    Procedure   LoadFromStream(Dimension: String);
    Function    CreateFromCSV(Dimension: String): Boolean;
//    Procedure   LoadHierarchieFromCSV(Dimension: String; sFileName: String);
{    Procedure   CheckUpdate;
    Function    Find(Value: String; LookIn: tlesMode; GiveBack: tlesMode): string;
    Function    IndexOf(iStart: Integer; Value: String; LookIn: tlesMode): Integer;
    procedure   EraseLike(value: String; LookIn: tlcgMode);
    Procedure   EraseRecord(Index: Integer);
}
    function    BinarySearch(Value: String): Integer;
    procedure   SortBy(Order: tlesSort);
    property    Items[i: Integer]: TElementSecurityRecord read GetData;
    property    count;
  end;

var lElementSecurity : TLElementSecurity;


implementation

uses rp_Functions;

function   TLElementSecurity.BinarySearch(Value: String): Integer;
var  sLookIn: String;
     relationship : integer;
     left : Integer;
     middle : Integer;
     right : Integer;
     found : Boolean;
//     index : Integer;
begin
  found := False;
  result := -1;

  left  := 0;
  right := Count - 1;

  while (left <= right) do
//  while (left <= right) and (not Found) do
  begin
    middle := (left + right) div 2;
    sLookIn := Items[middle].Element+Items[middle].Group;
    relationship := CompareStr(sLookIn, Value);
    if relationship < 0 then
       left := middle + 1
    else if relationship = 0 then
      begin
        result := middle;
        Break;
//        Found := True;
      end
      else
        right := middle - 1;
  end;
end;

function CompareElementAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementSecurityRecord(Item1^).Element, TElementSecurityRecord(Item2^).Element);
end;

function CompareElementDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementSecurityRecord(Item2^).Element, TElementSecurityRecord(Item1^).Element);
end;

function CompareGroupAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementSecurityRecord(Item1^).Group, TElementSecurityRecord(Item2^).Group);
end;

function CompareGroupDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TElementSecurityRecord(Item2^).Group, TElementSecurityRecord(Item1^).Group);
end;

function CompareBothAsc(Item1, Item2: Pointer): Integer;
var s1, s2: String;
begin
  Result := CompareText(TElementSecurityRecord(Item1^).Element + TElementSecurityRecord(Item1^).Group,TElementSecurityRecord(Item2^).Element + TElementSecurityRecord(Item2^).Group);
end;

function CompareBothDesc(Item1, Item2: Pointer): Integer;
var s1, s2: String;
begin
  s1 := TElementSecurityRecord(Item1^).Element + TElementSecurityRecord(Item1^).Group;
  s1 := TElementSecurityRecord(Item2^).Element + TElementSecurityRecord(Item2^).Group;
  Result := CompareText(s2, s1);
end;

procedure   TLElementSecurity.SortBy(Order: tlesSort);
begin
  case order of
    tlesElementAsc  : Sort(@CompareElementAsc);
    tlesElementDesc : Sort(@CompareElementDesc);
    tlesGroupAsc    : Sort(@CompareGroupAsc);
    tlesGroupDec    : Sort(@CompareGroupDesc);
    tlesBothAsc     : Sort(@CompareBothAsc);
    tlesBothDesc    : Sort(@CompareBothDesc);
  end;
  Sorted := Order;
end;

constructor TLElementSecurity.Create;
begin
  inherited create;
end;

destructor TLElementSecurity.Destroy;
begin
  inherited Destroy;
end;

function    TLElementSecurity.GetData(i: Integer): TElementSecurityRecord;
var p : PElementSecurityRecord;
begin
  p := inherited Items[i];
  result.Element   := p^.Element;
  result.Group     := p^.Group;
  result.Right     := p^.Right;
end;

procedure   TLElementSecurity.SetData(i: Integer; value: TElementSecurityRecord);
var p : PElementSecurityRecord;
begin
  p := inherited Items[i];
  p^.Element := value.Element;
  p^.Group   := value.Group;
  p^.Right   := p^.Right;
end;

function    TLElementSecurity.AddData(value: TElementSecurityRecord): Integer;
begin
  new(pElementSecurity);
  pElementSecurity^.Element := value.Element;
  pElementSecurity^.Group   := value.Group;
  pElementSecurity^.Right   := value.Right;
  inherited Add(pElementSecurity);
  result := count-1;
end;

procedure   TLElementSecurity.SaveToStream(Dimension: String);
var i, iLen, iCount, iArr: Integer;
    Data  : TElementSecurityRecord; // ein Zeiger auf die Datenstruktur
    s : String;
    sFileName: String;
begin

  sFileName := fPathProg + ElementSecurityFileStream + Dimension + ExtLst;

  Stream := TFileStream.Create(sFileName, fmCreate);

  iCount := Count;
  Stream.Write(iCount, SizeOf(iCount));

  For i := 0 to iCount - 1 do
  begin
    Data := Items[i];

    iLen := Length(Data.Element);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Element)^, iLen);

    iLen := Length(Data.Group);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Group)^, iLen);

    iLen := Length(Data.Right);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Right)^, iLen);

  end;
  stream.Free;
end;

Procedure   TLElementSecurity.LoadFromStream(Dimension: String);
var i, iLen, iCount, iArr: Integer;
    Data  : TElementSecurityRecord; // ein Zeiger auf die Datenstruktur
begin

  clearlist;

  Stream := TFileStream.Create(fPathProg + ElementSecurityFileStream + Dimension + ExtLst, fmOpenRead);

  Stream.Read(iCount, SizeOf(iCount));

  For i := 1 to iCount do
  begin

    FillChar(Data,SizeOf(Data),0);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Element, iLen);
    Stream.Read(PChar(Data.Element)^, iLen);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Group, iLen);
    Stream.Read(PChar(Data.Group)^, iLen);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Right, iLen);
    Stream.Read(PChar(Data.Right)^, iLen);

    AddData(Data);
  end;
  stream.Free;
end;

Procedure   TLElementSecurity.ClearList;
var p : PElementSecurityRecord;
    i : Integer;
begin
  for i:= Count-1 downto 0 do
  begin
    p:= inherited Items[i];
    Dispose(p);
    Delete(i);
  end;
end;

// '}ElementSecurity_*.cub';
Function    TLElementSecurity.CreateFromCSV(Dimension: String): Boolean;
var i: Integer;
    Data  : TElementSecurityRecord;
    Datum : TDateTime;
    tsl: TStringList;
    s, sFileName : String;
    sa: TStringArray;
begin
  result := False;
  sFileName := ElementSecurityFileCSV + Dimension + '.cub';

  if FileExists(fPathData + SFileName) then
  begin

    result := true;
    tsl := TStringList.Create;
    tsl.LoadFromFile(fPathData + sFileName);

    lElementSecurity.ClearList;
    for i := 0 to tsl.Count-1 do
    begin
      s := ConvertEncoding(tsl.Strings[i],GuessEncoding(tsl.Strings[i]), EncodingUTF8);
      sa := s.Split(';');
      Data.Element := sa[0];
      Data.Group   := sa[1];
      Data.Right   := sa[2];
      AddData(Data);
    end;
    tsl.Free;
    SortBy(tlesBothAsc);
//    SaveToStream(Dimension);
//    lAnwendungen.SearchReplace(Dimension,tlawDimension,DateTimeToStr(FileDateToDateTime(FileAge(fPathData + sFileName))),tlawLDatSecurity);
//    lElementSecurity.ClearList;
  end;
end;


initialization
if lElementSecurity = NIL then
begin
  lElementSecurity := TlElementSecurity.Create;
end;


finalization
if lElementSecurity <> NIL then
begin
  lElementSecurity.ClearList;
  lElementSecurity.Free;
end;

end.

