unit ulGroups;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LConvEncoding, uTypes, Math;

const
  GroupFileCSV = '}Groups.dim';
  GroupFileStream = 'Groups.lst';

type

{------------------------------------------------------------------------------}
{ Definition TM1-Group                                                         }
{ Quelle: Groups.dim                                                           }
{ Header: Vater;Kind;Attribute                                                 }
{ Satz:   ADMIN;ADMIN;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                      }
{         SecurityAdmin;SecurityAdmin;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      }
{------------------------------------------------------------------------------}


  tlgMode = (tlgElement,tlgVater,tlgKind,tlgState);
  tlgSort = (tlgVaterAsc,tlgVaterDesc,tlgKindAsc,tlgKindDesc,tlgOrderAsc,tlgOrderDesc);


  TGroupRecord = record
    Name_Vater : String;  { Key Vater-Element                                  }
    Name_Kind  : String;  { Key Kind-Element                                   }
    Order      : Integer; { Index beim Einlesen                                }
  end;

    PGroupRecord = ^TGroupRecord;

  TLGroup = class(TList)
  private
    pGroup      : ^TGroupRecord;
    Stream     : TStream;
    Sorted     : tlgSort;
    Datum      : TDateTime;   // Letztes Ladedatum mit Zeit
  protected
    function    AddData(value: TGroupRecord): Integer;
    function    GetData(i: Integer): TGroupRecord;
    procedure   SetData(i: Integer; value: TGroupRecord);
    procedure   SaveToStream;
    Procedure   ClearList;
  public
    constructor Create;
    destructor  Destroy; override;
    Procedure   LoadFromStream;
    Procedure   LoadFromCSV;
    procedure   AddValues(Vater: String; Kind: String);
    procedure   SetValues(Index: Integer; Vater: String; Kind: String);
    Procedure   Erase(Index: Integer);
    Function    IndexOf(iStart: Integer; Value: String; LookIn: tlgMode): Integer;
    procedure   SortBy(Order: tlgSort);
    property    Items[i: Integer]: TGroupRecord read GetData;
    property    count;
  end;

var lGroups : TLGroup;

implementation

uses ufmain, rp_Functions;

function CompareVaterAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TGroupRecord(Item1^).Name_Vater, TGroupRecord(Item2^).Name_Vater);
end;

function CompareVaterDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TGroupRecord(Item2^).Name_Vater, TGroupRecord(Item1^).Name_Vater);
end;

function CompareKindAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TGroupRecord(Item1^).Name_Kind, TGroupRecord(Item2^).Name_Kind);
end;

function CompareKindDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TGroupRecord(Item2^).Name_Kind, TGroupRecord(Item1^).Name_Kind);
end;

function CompareOrderAsc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TGroupRecord(Item1^).Order, TGroupRecord(Item2^).Order);
end;

function CompareOrderDesc(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TGroupRecord(Item2^).Order, TGroupRecord(Item1^).Order);
end;

procedure   TLGroup.SortBy(Order: tlgSort);
begin
  case Order of
    tlgVaterAsc  : Sort(@CompareVaterAsc);
    tlgVaterDesc : Sort(@CompareVaterDesc);
    tlgKindAsc   : Sort(@CompareKindAsc);
    tlgKindDesc  : Sort(@CompareKindDesc);
    tlgOrderAsc  : Sort(@CompareOrderAsc);
    tlgOrderDesc : Sort(@CompareOrderDesc);
  end;
  Sorted := Order;
end;


constructor TLGroup.Create;
begin
  inherited create;
end;

destructor TLGroup.Destroy;
begin
  inherited Destroy;
end;

Procedure   TLGroup.Erase(Index: Integer);
var p : PGroupRecord;
begin
  if (Index > -1) and (Index < count) then
  begin
    p := inherited Items[Index];
    Dispose(p);
    inherited Delete(Index);
  end;
end;

Function    TLGroup.IndexOf(iStart: Integer; Value: String; LookIn: tlgMode): Integer;
var i : Integer;
    boFound : Boolean;
begin
  result := -1;
  boFound := False;
  for i := iStart to count - 1 do
  begin
    case LookIn of
      tlgElement : boFound := (GetData(i).Name_Vater = Value) or (GetData(i).Name_Kind = Value);
      tlgVater   : boFound := GetData(i).Name_Vater = Value;
      tlgKind    : boFound := GetData(i).Name_Kind = Value;
    end;

    if boFound then
    begin
      result := i;
      Break;
    end;
  end;
end;

procedure   TLGroup.SetValues(Index: Integer; Vater: String; Kind: String);
var Data: TGroupRecord;
begin
  Data.Name_Vater := Vater;
  Data.Name_Kind  := Kind;
  SetData(Index,Data);
end;

procedure   TLGroup.AddValues(Vater: String; Kind: String);
var Data: TGroupRecord;
begin
  Data.Name_Vater := Vater;
  Data.Name_Kind  := Kind;
  Data.Order      := Count+1;
  AddData(Data);
end;

function    TLGroup.AddData(value: TGroupRecord): Integer;
begin
  new(pGroup);
  pGroup^.Name_Vater  := value.Name_Vater;
  pGroup^.Name_Kind   := value.Name_Kind;
  pGroup^.Order       := value.Order;
  inherited Add(pGroup);
  result := count-1;
end;

function    TLGroup.GetData(i: Integer): TGroupRecord;
var p : PGroupRecord;
begin
  p := inherited Items[i];
  result.Name_Vater  := p^.Name_Vater;
  result.Name_Kind   := p^.Name_Kind;
  result.Order       := p^.Order;
end;

procedure   TLGroup.SetData(i: Integer; value: TGroupRecord);
var p : PGroupRecord;
begin
  p := inherited Items[i];
  p^.Name_Vater := value.Name_Vater;
  p^.Name_Kind  := value.Name_Kind;
  p^.Order      := value.Order;
end;

procedure   TLGroup.SaveToStream;
var i, iLen, iCount: Integer;
    Data  : TGroupRecord; // ein Zeiger auf die Datenstruktur
    sDatum: String;
begin
  Stream := TFileStream.Create(fPathProg + GroupFileStream, fmCreate);

  iCount := Count;
  sDatum := DateTimeTostr(Datum);

  Stream.Write(iCount, SizeOf(iCount));

  Stream.Write(Sorted, SizeOf(Sorted));

  Stream.Write(Datum, SizeOf(Datum));

  For i := 0 to iCount - 1 do
  begin
    Data := Items[i];
    iLen := Length(Data.Name_Vater);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Name_Vater)^, iLen);

    iLen := Length(Data.Name_Kind);
    Stream.Write(iLen, SizeOf(iLen));
    Stream.Write(PChar(Data.Name_Kind)^, iLen);

    Stream.Write(Data.Order, SizeOf(Data.Order));

  end;
  stream.Free;

end;

Procedure   TLGroup.LoadFromStream;
var i, iLen, iCount: Integer;
    Data  : TGroupRecord; // ein Zeiger auf die Datenstruktur
    sDatum : String;
begin

  clearlist;
  Stream := TFileStream.Create(fPathProg + GroupFileStream, fmOpenRead);

  Stream.Read(iCount, SizeOf(iCount));

  Stream.Read(Sorted, SizeOf(Sorted));

  Stream.Read(Datum, SizeOf(Datum));

  For i := 1 to iCount do
  begin

    FillChar(Data,SizeOf(Data),0);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Name_Vater, iLen);
    Stream.Read(PChar(Data.Name_Vater)^, iLen);

    Stream.Read(iLen, SizeOf(iLen));
    SetLength(Data.Name_Kind, iLen);
    Stream.Read(PChar(Data.Name_Kind)^, iLen);

    Stream.Read(Data.Order, SizeOf(Data.Order));

    AddData(Data);
  end;
  stream.Free;

  if FileDateToDateTime(FileAge(fPathData + GroupFileCSV)) > Datum then LoadFromCSV;
end;

Procedure   TLGroup.LoadFromCSV;
var i: Integer;
    s: String;
    Data  : TGroupRecord;
    tsl: TStringList;
    sa: TStringArray;
begin
  clearlist;
  tsl := TStringList.Create;

  tsl.LoadFromFile(fPathData + GroupFileCSV);

  Datum := FileDateToDateTime(FileAge(fPathData + GroupFileCSV));
  Sorted := tlgOrderAsc;

  for i := 1 to tsl.Count-1 do
  begin
     s := ConvertEncoding(tsl.Strings[i],GuessEncoding(tsl.Strings[i]), EncodingUTF8);
     sa := s.Split(';');
     Data.Name_Vater := sa[0];
     Data.Name_Kind  := sa[1];
     Data.Order := i;
     AddData(Data);
  end;
  tsl.Clear;
  tsl.Free;
  setLength(sa,0);
end;

Procedure   TLGroup.ClearList;
var p : PGroupRecord;
begin
  while count > 0 do
  begin
    p := inherited Items[0];
    Dispose(p);
    Delete(0);
  end;
end;


initialization
if lGroups = NIL then
begin
  lGroups := TlGroup.Create;
end;

finalization
if lGroups <> NIL then
begin
  lGroups.SaveToStream;
  lGroups.ClearList;
  lGroups.Free;
end;


end.

